<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
</head>
<body>
    <h1>Microsoft Identity Python Web App</h1>

    <li><a href='{{ auth_url }}'>Sign In</a></li>

    <hr>
    <footer style="text-align: right">Powered by MSAL Python {{ version }}</footer>
</body>
</html>

